//#include "metalShaderTypes.h"

typedef struct
{
	float blurAmount;
} fxVars;

typedef struct
{
	float4 position [[position]];
	float2 tex1;
	float2 texMask;
	float2 normPos;
	float2 texSize;
	float2 texCoords0;
	float2 texCoords1;
	float2 texCoords2;
	float2 texCoords3;
	float2 texCoords4;
	float2 texCoords5;
	float2 texCoords6;
	float2 texCoords7;
	float2 texCoords8;
	float2 texCoords9;
	float2 texCoords10;
	float2 texCoords11;
	float2 texCoords12;
	float2 texCoords13;
	float2 texCoords14;
} fsVertexOut;

constant int samples = 50,
          LOD = 2,         // gaussian done on MIPmap at scale LOD
          sLOD = 1 << LOD; // tile size = 2^LOD
constant float sigma = float(samples) * .25;

float gaussian(vec2 i) {
    return exp( -.5* dot(i/=sigma,i) ) / ( 6.28 * sigma*sigma );
}

float4 blur(texture2d<half> inputTex1, float2 U, float2 scale)
{
    float4 O = float4(0);
    int s = samples/sLOD;
    
    for ( int i = 0; i < s*s; i++ )
    {
        float2 d = float2(i%s, i/s)*float(sLOD) - float(samples)/2.;
        O += gaussian(d) * float4(inputTex1.sample(linearSampler,U + scale * d , level(5) ));
    }
    
    return O / O.a;
}

vec3 draw(texture2d<half> inputTex1,vec2 uv)
{
//    return texture(iChannel0,vec2(uv.x,1.-uv.y)).rgb;
    //return texture(iChannel0,uv).rgb;
    float4 color=float4(inputTex1.sample(linearSampler,vec2(uv.x,uv.y)));
    return color.rgb;
}

float grid(float var, float size) {
    return floor(var*size)/size;
}

float rand(vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}
float degrees(float a)
{
    return (a*180.0)/3.14159265359;
}

float4 blur2(texture2d<half> inputTex1,vec2 uv,float2 scale )
{
    vec4 fragColor;
//    float time = iTime;
    
//    float bluramount = sin(time)*0.1;
//    if (iMouse.w >= 1.) {
//    bluramount = (iMouse.x/iResolution.x)/10.;
//    }
    float bluramount=.04;

    //float dists = 5.;
    vec3 blurred_image = vec3(0.);
    #define repeats 40.
    for (float i = 0.; i < repeats; i++) {
        //Older:
        //vec2 q = vec2(cos(degrees((grid(i,dists)/repeats)*360.)),sin(degrees((grid(i,dists)/repeats)*360.))) * (1./(1.+mod(i,dists)));
        vec2 q = vec2(cos(degrees((i/repeats)*360.)),sin(degrees((i/repeats)*360.))) *  (rand(vec2(i,uv.x+uv.y))+(bluramount*.014));
        vec2 uv2 = uv+(q*bluramount);//*scale;
        blurred_image += draw(inputTex1,uv2)/2.;
        //One more to hide the noise.
        q = vec2(cos(degrees((i/repeats)*360.)),sin(degrees((i/repeats)*360.))) *  (rand(vec2(i+2.,uv.x+uv.y+24.))+(bluramount*.012));
        uv2 = uv+(q*bluramount);//*scale;
        blurred_image += draw(inputTex1,uv2)/2.;
    }
    blurred_image /= repeats;
        
    fragColor = vec4(blurred_image,1.0);
    return fragColor;
}

#define QUALITY 32


constant float GOLDEN_ANGLE = 2.399963229728653; // PI * (3.0 - sqrt(5.0))

vec4 blur3(texture2d<half> tpipe,vec2 c,float radius)
{
     vec4 sum = vec4(0.0);
     float q = float(QUALITY);
     // Using a "spiral" to propagate points.
     for (int i=0; i<QUALITY; ++i)
     {
          float fi = float(i);
          float a = fi * GOLDEN_ANGLE;
          float r = sqrt(fi / q) * radius;
          vec2 p = c + r * vec2(cos(a), sin(a));
          sum += tex(tpipe, p);
     }
     return sum / q;
}

vec4 blur13(texture2d<half> image, vec2 uv, vec2 scale, vec2 direction)
{
  vec4 color = vec4(0.0);
  vec2 off1 = vec2(1.411764705882353) * direction;
  vec2 off2 = vec2(3.2941176470588234) * direction;
  vec2 off3 = vec2(5.176470588235294) * direction;
  color += float4(image.sample(linearSampler, uv)) * 0.1964825501511404;
  color += float4(image.sample(linearSampler, uv + (off1 * scale))) * 0.2969069646728344;
  color += float4(image.sample(linearSampler, uv - (off1 * scale))) * 0.2969069646728344;
  color += float4(image.sample(linearSampler, uv + (off2 * scale))) * 0.09447039785044732;
  color += float4(image.sample(linearSampler, uv - (off2 * scale))) * 0.09447039785044732;
  color += float4(image.sample(linearSampler, uv + (off3 * scale))) * 0.010381362401148057;
  color += float4(image.sample(linearSampler, uv - (off3 * scale))) * 0.010381362401148057;
  return color;
}

float4 blur4(texture2d<half> inputTex1,vec2 uv,float2 scale)
{
    float step=1.;
    float2 direction=float2(step,0.0);
    float4 outColor;
    outColor=blur13(inputTex1,uv,scale,direction);
    direction=float2(0.0,step);
    outColor+=blur13(inputTex1,uv,scale,direction);
    outColor*=.5;
    return outColor;
}
    

float4 fetchTexture(texture2d<half> inputTex0,texture2d<half> inputTex1,float2 texCoord1,float2 texCoord2,float saturation)
{
    float4 color1=float4(inputTex0.sample(linearSampler,texCoord1));
//    float4 color3=blur3(inputTex1,texCoord2,.02);


    float4 color3=blur2(inputTex1,texCoord2,1./float2(inputTex1.get_width(),inputTex1.get_height()));
    float4 color2=float4(inputTex1.sample(linearSampler,texCoord2));
    color3=mix(color3,color2,0.0);
//    float4 color3=float4(1.0)-color2;
//    color3.a=color2.a;
    
    color3*=color1.a;
//    return(mix(color1,color3,1.));
    return color3;
}


vertex fsVertexOut vertexFunc(uint vertexID [[ vertex_id ]]
							  ,const device fxShaderVerts* in[[ buffer(0) ]]
							  ,texture2d<half> inputTex0 [[ texture(0) ]]
							  ,texture2d<half> inputTexMask [[ texture(1) ]]
							  ,constant fxGeneralUniforms& u[[ buffer(1) ]]
							  ,constant fxVars& vars[[ buffer(2)]])
{
    fsVertexOut out;

	out.normPos=in[vertexID].pos.xy*vec2(.5,.5f)+vec2(.5,.5);
	out.position=in[vertexID].pos;

    float2 texCoord=out.normPos;

    out.tex1=in[vertexID].tex1;
	out.texMask=in[vertexID].texMask;
	out.texSize=float2(inputTex0.get_width(),inputTex0.get_height());
	
	float fsScale=vars.blurAmount*3.0;
	if(fsScale==0.0)
		fsScale=0.1;

	float2 direction;
	float pass=u.pass;
	if(pass>1)
		pass-=2;
	if(pass==0)
	{
		direction=vec2((fsScale+fsScale*.5)/out.texSize.x,0.0);
	}
	else
	{
		direction=vec2(0.0,(fsScale+fsScale*.5)/out.texSize.y);
	}

	out.texCoords0 = texCoord + vec2(-7.0,-7.0)*direction;
	out.texCoords1 = texCoord + vec2(-6.0,-6.0)*direction;
	out.texCoords2 = texCoord + vec2(-5.0,-5.0)*direction;
	out.texCoords3 = texCoord + vec2(-4.0,-4.0)*direction;
	out.texCoords4 = texCoord + vec2(-3.0,-3.0)*direction;
	out.texCoords5 = texCoord + vec2(-2.0,-2.0)*direction;
	out.texCoords6 = texCoord + vec2(-1.0,-1.0)*direction;
	out.texCoords7 = texCoord + vec2( 1.0, 1.0)*direction;
	out.texCoords8 = texCoord + vec2( 2.0, 2.0)*direction;
	out.texCoords9 = texCoord + vec2( 3.0, 3.0)*direction;
	out.texCoords10 = texCoord + vec2( 4.0, 4.0)*direction;
	out.texCoords11 = texCoord + vec2( 5.0, 5.0)*direction;
	out.texCoords12 = texCoord + vec2( 6.0, 6.0)*direction;
	out.texCoords13 = texCoord + vec2( 7.0, 7.0)*direction;
	out.texCoords14 = texCoord;
    return out;
}

fragment float4 fragmentFunc(fsVertexOut input [[stage_in]]
							 ,texture2d<half> inputTex [[ texture(0) ]]
							 ,texture2d<half> inputTexMask [[ texture(1) ]]
							 ,constant fxGeneralUniforms& u[[ buffer(0) ]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
	
	vec4 texel = tex(inputTex, fsTexture);
	float4 outColor=vec4(0.0);
	
    outColor += tex(inputTex, input.texCoords0);
    outColor += tex(inputTex, input.texCoords1);
    outColor += tex(inputTex, input.texCoords2);
    outColor += tex(inputTex, input.texCoords3);
    outColor += tex(inputTex, input.texCoords4);
    outColor += tex(inputTex, input.texCoords5);
    outColor += tex(inputTex, input.texCoords6);
    outColor += tex(inputTex, input.texCoords14);
    outColor += tex(inputTex, input.texCoords7);
    outColor += tex(inputTex, input.texCoords8);
    outColor += tex(inputTex, input.texCoords9);
    outColor += tex(inputTex, input.texCoords10);
    outColor += tex(inputTex, input.texCoords11);
    outColor += tex(inputTex, input.texCoords12);
    outColor += tex(inputTex, input.texCoords13);

    outColor*=vec4(1.0/15.0,1.0/15.0,1.0/15.0,1.0/15.0);

	float blendValue=u.blendValue;
#ifdef USE_MASK
	blendValue*=texture(maskTex,fsMaskTexture).a;
#endif
    outColor.rgb=mix(texel.rgb,outColor.rgb,blendValue);
	return outColor;
}
