#version 330

uniform sampler2DRect inputTex0;
uniform sampler2DRect inputTexMask;
uniform vec4 endColor;
uniform float endOpacity;

in vec2 fsTexture;
in vec2 fsTextureMask;
out vec4 outColor;

#ifdef AdobeRGB1998
    const vec4  profileToRGB_R = vec4(1.398283,-0.398283064, 0.0,0.0  );
    const vec4  profileToRGB_G = vec4(0.0,1.0,0.0,0.0  );
    const vec4  profileToRGB_B = vec4(0.0, -0.0429382771,1.04293835,0.0  );
#endif

#ifdef DISPLAY_P3
    const vec4  profileToRGB_R = vec4(1.2249,-0.2247, 0.0,0.0  );
    const vec4  profileToRGB_G = vec4(-0.0420,1.0419,0.0,0.0  );
    const vec4  profileToRGB_B = vec4(-0.0197, -0.0786,1.0979,0.0  );
#endif

vec4 toGamma(vec4 linearRGB)
{
    bvec3 cutoff = lessThan(linearRGB.rgb, vec3(0.0031308));
    vec3 higher = sqrt(linearRGB.rgb);
    vec3 lower = linearRGB.rgb * vec3(12.92);

    return vec4(mix(higher, lower, cutoff), linearRGB.a);
}

// Converts a color from sRGB gamma to linear light gamma
vec4 toLinear(vec4 sRGB)
{
    bvec3 cutoff = lessThan(sRGB.rgb, vec3(0.04045));
    vec3 higher = pow((sRGB.rgb + vec3(0.055))/vec3(1.055), vec3(2.4));
    vec3 lower = sRGB.rgb/vec3(12.92);

    return vec4(mix(higher, lower, cutoff), sRGB.a);
}

void main()
{
    
#ifdef ARGB
    outColor=texture(inputTex0, fsTexture).argb;
#endif
    
#ifdef ABGR
    outColor=texture(inputTex0, fsTexture).abgr;
#endif

#ifdef BGRA
    outColor=texture(inputTex0, fsTexture).bgra;
#endif
    
#ifdef RGBA
    outColor=texture(inputTex0, fsTexture).rgba;
#endif
    
#ifdef FORCE_ALPHA
    outColor.a=1.0;
#endif
    
#ifndef NO_PREMULT
    outColor.rgb*=outColor.a;
#endif

#ifdef USE_COLOR_PROFILE
    vec4 linear=toLinear(outColor);

    vec4 srgbLinear=vec4(dot(profileToRGB_R, linear),
                    dot(profileToRGB_G, linear),
                    dot(profileToRGB_B, linear),
                    outColor.a);

    outColor=toGamma(srgbLinear);
#endif


#ifdef USE_MASK
    outColor.a*=texture(inputTexMask,fsTextureMask).a;
#endif

#ifdef USE_END_COLOR
    outColor=mix(endColor,outColor,endOpacity);
#endif


}
