#include "inputShaderHelperDX.h"

v2p vertexShader(vertexInfo input)
{
    v2p output;
    output.position = float4(input.position,1.0);
    output.texCoordY = input.texCoordY;
    output.texCoordUV = input.texCoordY;
    return output;
}

float4 pixelShader(v2p input) : SV_TARGET
{
    float4 yuvIn;

    float width,height;
    inputTex0.GetDimensions(width,height);
    float2 normalizedTC=input.texCoordY/float2(width,height);
    
    yuvIn.g = inputTex0.Sample(samplerInputTex0, normalizedTC).r;
    yuvIn.b = inputTex1.Sample(samplerInputTex1, normalizedTC).r;
    yuvIn.r = inputTex2.Sample(samplerInputTex2, normalizedTC).r;
    yuvIn.a=0.0;

#ifdef HAS_ALPHA
    yuvIn.a = inputTex3.Sample(samplerInputTex3, normalizedTC).r;
#endif

#ifdef IS_10BIT
    yuvIn*=float4(64.0, 64.0, 64.0, 64.0);;
#endif
    
#ifdef IS_12BIT
    yuvIn*=float4(16.0, 16.0, 16.0, 16.0);
#endif
    
    yuvIn+=stdbias;

    float4 outColor=float4(dot(yuv2R, yuvIn),
                  dot(yuv2G, yuvIn),
                  dot(yuv2B, yuvIn),
                  1.0);

#ifdef USE_COLOR_PROFILE
    float4 linearRGB=toLinear(outColor);

    float4 srgbLinear=float4(dot(profileToRGB_R, linearRGB),
        dot(profileToRGB_G, linearRGB),
        dot(profileToRGB_B, linearRGB),
        outColor.a);

    outColor=toGamma(srgbLinear);
#endif

#ifndef NO_PREMULT
    outColor.rgb*=outColor.a;
#endif

#ifdef USE_MASK
    outColor.a=maskTex.Sample(samplerMaskTex,normalizedTC).a;
#endif

#ifdef USE_END_COLOR
    outColor=mix(uniforms.endColor,outColor,uniforms.endOpacity);
#endif

    return outColor;
}
