#include "dxShaderIncludes.h"


typedef struct
{
	float zoom;
	float size;
	float colorSeparation;
	float pad;
} fxVars;

cbuffer variableBuffer : register(b2)
{
	fxVars vars;
}

vertexOut vertexShader(txShaderVerts input)
{
	vertexOut output;
	output.position = mul(u.worldViewProj,float4(input.position,1));
	output.tex1 = input.tex1;
	output.tex2 = input.tex2;
	output.texMask = input.texMask;
	return output;
}

float4 pixelShader(vertexOut input) : SV_TARGET
{
	float inv = 1. - u.transitionProgressLinear;
	vec2 disp = vars.size*vec2(cos(vars.zoom*input.tex1.x), sin(vars.zoom*input.tex1.y));
	vec2 disp2=vars.size*vec2(cos(vars.zoom*input.tex2.x), sin(vars.zoom*input.tex2.y));

	vec4 colorIn=tex(inputTex0,(input.tex1+inv*disp));

	float progress=u.transitionProgressLinear;
	vec4 colorOut;

	colorOut.r=tex(inputTex1,(input.tex2+progress*disp2*(1.0-vars.colorSeparation))).r;
	colorOut.ga=tex(inputTex1,(input.tex2+progress*disp2)).ga;
	colorOut.b=tex(inputTex1,(input.tex2+progress*disp2*(1.0+vars.colorSeparation))).b;

	float4 outColor=mix(colorOut,colorIn,u.transitionProgress);
#ifdef USE_MASK
	outColor.a*=float(inputTexMask.sample(linearSampler,input.texMask).a);
#endif

	return outColor;
}
