#include "dxShaderIncludes.h"


typedef struct
{
	float4 burnColor;
	float pad[3];
} fxVars;

cbuffer variableBuffer : register(b2)
{
	fxVars vars;
}

vertexOut vertexShader(txShaderVerts input)
{
	vertexOut output;
	output.position = mul(u.worldViewProj,float4(input.position,1));
	output.tex1 = input.tex1;
	output.tex2 = input.tex2;
	output.texMask = input.texMask;
	output.normPos=input.position.xy*vec2(.5,.5f)+vec2(.5,.5);
	return output;
}

vec4 grayscale (vec4 color)
{
	float gray=0.2126*color.r + 0.7152*color.g + 0.0722*color.b;
	return vec4(gray,gray,gray,color.a);
}

float4 pixelShader(vertexOut input) : SV_TARGET
{
	vec4 colorA = tex(inputTex0, input.tex1);
	vec4 colorB = tex(inputTex1, input.tex2);
	vec4 color1=mix(grayscale(colorA),colorA,smoothstep(.6,1.0,u.transitionProgress));
	vec4 color2=mix(grayscale(colorB),colorB,smoothstep(.6,1.0,1.-u.transitionProgress));
	float4 outColor=mix(color2,color1,u.transitionProgress);

	#ifdef USE_MASK
	outColor.a*=float(inputTexMask.sample(linearSampler,input.texMask).a);
	#endif

	return outColor;
}
