#include "dxShaderIncludes.h"


typedef struct
{
	int direction;
	float pad[3];
} fxVars;

cbuffer variableBuffer : register(b2)
{
	fxVars vars;
}

static const vec2 fsDirectionTable[9] = {
	vec2(1,.5),
	vec2(0,.5),
	vec2(-1,.5),
	vec2(1,0),
	vec2(0,0),
	vec2(-1,0),
	vec2(1,-.5),
	vec2(0,-.5),
	vec2(-1,-.5)
};


vertexOut vertexShader(txShaderVerts input)
{
	vertexOut output;

	output.tex1 = input.tex1;
	output.tex2 = input.tex2;
	output.texMask = input.texMask;
	output.position=mul(u.worldViewProj,float4(input.position,1));
	output.normPos=input.position.xy*vec2(.5,.5f)+vec2(.5,.5);

	return output;
}


static const vec2 center = vec2(0.5, 0.5);

float4 pixelShader(vertexOut input) : SV_TARGET
{

	vec2 fsDirection=normalize(fsDirectionTable[vars.direction]);
	vec2 p = input.normPos;
	vec2 v = fsDirection;
	if(v.x==0.0)
	v.x+=.01;
	if(v.y==0.0)
	v.y+=.01;

	v /= abs(v.x)+abs(v.y);
	float d = v.x * center.x + v.y * center.y;
	float offset = 1.0;
	float pr = smoothstep(-offset, 0.0, v.x * p.x + v.y * p.y - (d-0.5+u.transitionProgressLinear*(1.+offset)));
	vec2 squarep = fract(p*vec2(10,10));
	vec2 squaremin = vec2(pr/2.0,pr/2.0);
	vec2 squaremax = vec2(1.0 - pr/2.0,1.0 - pr/2.0);
	float a = all(squaremin<squarep) && all(squarep<squaremax) ? 1.0 : 0.0;
	vec4 texelA=tex(inputTex0,input.tex1);
	vec4 texelB=tex(inputTex1,input.tex2);
	float4 outColor=mix(texelB,texelA,a);

	#ifdef USE_MASK
	outColor.a*=float(inputTexMask.sample(linearSampler,input.texMask).a);
	#endif

	return outColor;
}
