#include "dxShaderIncludes.h"

typedef struct
{
    int direction;
    float pad[3];
} fxVars;

cbuffer variableBuffer : register(b2)
{
    fxVars vars;
}

vertexOut vertexShader(txShaderVerts input)
{
    vertexOut output;
    output.position = mul(u.worldViewProj,float4(input.position,1));
    output.tex1 = input.tex1;
    output.tex2 = input.tex2;
    output.texMask = input.texMask;
    output.normPos=input.position.xy*vec2(.5,.5f)+vec2(.5,.5);
    return output;
}


float4 pixelShader(vertexOut input) : SV_TARGET
{
    vec4 colorA = tex(inputTex0, input.tex1)*u.baseAlpha;
    vec4 colorB = tex(inputTex1, input.tex2)*u.baseAlpha;

    float diffIn;
    float diffOut;

    switch(vars.direction)
    {
        case 0: //top left
        {
            float tp=u.transitionProgress*1.1;

            diffIn=tp-(input.tex1.x+input.tex1.y)*.5;
            diffOut=(input.tex1.x+input.tex1.y)*.5-(tp-.1);
            break;
        }
        case 1: //top center
        {
            float tp=u.transitionProgress*1.1;

            diffIn=tp-input.tex1.y;
            diffOut=input.tex1.y-(tp-.1);
            break;
        }
        case 2: //top right
        {
            float tp=u.transitionProgress*1.1;
            diffIn=tp-((1.0-input.tex1.x)+input.tex1.y)*.5;
            diffOut=((1.0-input.tex1.x)+input.tex1.y)*.5-(tp-.1);
            break;
            break;
        }
        case 3: //middle left
        {
            float tp=u.transitionProgress*1.1;

            diffIn=tp-input.tex1.x;
            diffOut=input.tex1.x-(tp-.1);
            break;
        }
        case 4: //middle center - not used
        {
            float tp=u.transitionProgress*1.1;

            diffIn=tp-input.tex1.x;
            diffOut=input.tex1.x-(tp-.1);
            break;
        }
        case 5: //middle right
        {
            float tp=(1.0-u.transitionProgress)*1.1;

            diffIn=input.tex1.x-(tp-.1);
            diffOut=tp-input.tex1.x;
            break;
        }
        case 6: //bottom left
        {
            float tp=u.transitionProgress*1.1;

            diffIn=tp-(input.tex1.x+(1.0-input.tex1.y))*.5;
            diffOut=(input.tex1.x+(1.0-input.tex1.y))*.5-(tp-.1);
            break;
        }
        case 7: //bottom center
        {
            float tp=(1.0-u.transitionProgress)*1.1;

            diffIn=input.tex1.y-(tp-.1);
            diffOut=tp-input.tex1.y;
            break;
        }
        case 8: //bottom right
        {
            float tp=u.transitionProgress*1.1;
            vec2 tmp=1.0-input.tex1;
            diffIn=tp-(tmp.x+tmp.y)*.5;
            diffOut=(tmp.x+tmp.y)*.5-(tp-.1);
            break;
        }
    }


    diffIn=clamp(diffIn*10.0,0.0,1.0);
    diffOut=clamp(diffOut*10.0,0.0,1.0);
    float4 outColor=colorA*diffIn+colorB*diffOut;;

    #ifdef USE_MASK
    outColor.a*=float(inputTexMask.sample(linearSampler,input.texMask).a);
    #endif

    return outColor;
}
